@@ -1,18 +1,31 @@
 import pytest
+import pytest_asyncio
 import asyncio
 from unittest.mock import MagicMock, patch
 from victron_mqtt._victron_enums import DeviceType, GenericOnOff
-from victron_mqtt.data_classes import TopicDescriptor
 from victron_mqtt.hub import Hub
 from victron_mqtt.constants import TOPIC_INSTALLATION_ID
 from victron_mqtt.switch import Switch
 from victron_mqtt._victron_topics import topics
+import json
+import logging
 
-@pytest.fixture
-async def create_mocked_hub():
+# Configure logging for the test
+logging.basicConfig(level=logging.DEBUG)
+logger = logging.getLogger(__name__)
+
+@pytest_asyncio.fixture
+async def create_mocked_hub_with_installation_id() -> Hub:
+    return await create_mocked_hub_internal(installation_id="123")
+
+@pytest_asyncio.fixture
+async def create_mocked_hub() -> Hub:
+    return await create_mocked_hub_internal()
+
+async def create_mocked_hub_internal(installation_id=None) -> Hub:
     """Helper function to create and return a mocked Hub object."""
     with patch('victron_mqtt.hub.mqtt.Client') as mock_client:
-        hub = Hub(host="localhost", port=1883, username=None, password=None, use_ssl=False)
+        hub = Hub(host="localhost", port=1883, username=None, password=None, use_ssl=False, installation_id = installation_id)
         mocked_client = mock_client.return_value
 
         # Set the mocked client explicitly to prevent overwriting
@@ -42,69 +55,81 @@ async def create_mocked_hub():
                     None,
                     MagicMock(topic="N/123/system/0/Serial", payload=b'{"value": "123"}')
                 )
-        setattr(hub, '_subscribe', MagicMock(name="_subscribe", side_effect=mock_subscribe))
-
-        # Mock publish to set an event when the keepalive topic is received
-        keepalive_event = asyncio.Event()
+        mocked_client.subscribe = MagicMock(name="subscribe", side_effect=mock_subscribe)
 
         def mock_publish(topic, value):
             if topic == "R/123/keepalive":
-                keepalive_event.set()
-        setattr(hub, 'publish', MagicMock(name="publish", side_effect=mock_publish))
+                echo = parse_keepalive_options(value)
+                keepalive_payload = json.dumps({"full-publish-completed-echo": echo, "value": 42})
+                mocked_client.on_message(
+                    mocked_client,
+                    None,
+                    MagicMock(topic="N/123/full_publish_completed", payload=keepalive_payload.encode())
+                )
+        mocked_client.publish = MagicMock(name="publish", side_effect=mock_publish)
 
-        # Dynamically add the keepalive_event attribute to the Hub instance
-        setattr(hub, 'keepalive_event', keepalive_event)
+        await hub.connect()
 
-        # Run connect as an asyncio task
-        connect_task = asyncio.create_task(hub.connect())
+        return hub
+
+def parse_keepalive_options(json_string: str) -> str:
+    """Parse the JSON string for keepalive options and return the echo value."""
+    try:
+        options = json.loads(json_string)
+        keepalive_options = options.get("keepalive-options", [])
+        if keepalive_options and isinstance(keepalive_options, list):
+            return keepalive_options[0].get("full-publish-completed-echo", "")
+        return ""
+    except (json.JSONDecodeError, AttributeError, IndexError):
+        return ""
 
-        # Wait for the keepalive event before returning the hub
-        await keepalive_event.wait()
 
-        return hub, connect_task
 
 def inject_message(hub_instance, topic, payload):
     """Helper function to inject a single MQTT message into the Hub."""
     hub_instance._client.on_message(None, None, MagicMock(topic=topic, payload=payload.encode()))
 
+async def finalize_injection(hub: Hub, disconnect: bool = True):
+    """Finalize the injection of messages into the Hub."""
+    # Wait for the connect task to finish
+    await hub._keepalive()
+    await hub.wait_for_first_refresh()
+    if disconnect:
+        await hub.disconnect()
+
 @pytest.mark.asyncio
 async def test_hub_initialization(create_mocked_hub):
     """Test that the Hub initializes correctly."""
-    hub, _ = await create_mocked_hub
+    hub: Hub = create_mocked_hub
     assert hub._client is not None, "MQTT client should be initialized"
 
 @pytest.mark.asyncio
 async def test_hub_message_handling(create_mocked_hub):
     """Test that the Hub processes incoming MQTT messages correctly."""
-    hub, _ = await create_mocked_hub
+    hub: Hub = create_mocked_hub
 
     # Inject a message
     inject_message(hub, "N/device/123/metric/456", "{\"value\": 42}")
 
     # Validate the Hub's state
-    assert len(hub._devices) == 0, "No devices should be created"
+    assert len(hub._devices) == 1, "No devices should be created"
 
-async def finalize_injection(hub, connect_task):
-    """Finalize the injection of messages into the Hub."""
-    inject_message(hub, "full_publish_completed", "1")
-    # Wait for the connect task to finish
-    await connect_task
-    await hub.disconnect()
+    await finalize_injection(hub)
 
 @pytest.mark.asyncio
 async def test_phase_message(create_mocked_hub):
     """Test that the Hub correctly updates its internal state based on MQTT messages."""
-    hub, connect_task = await create_mocked_hub
+    hub: Hub = create_mocked_hub
 
     # Inject messages after the event is set
     inject_message(hub, "N/123/grid/30/Ac/L1/Energy/Forward", "{\"value\": 42}")
-    await finalize_injection(hub, connect_task)
+    await finalize_injection(hub)
 
     # Validate the Hub's state
-    assert len(hub._devices) == 1, f"Expected 1 device, got {len(hub._devices)}"
+    assert len(hub._devices) == 2, f"Expected 2 device, got {len(hub._devices)}"
 
     # Validate that the device has the metric we published
-    device = list(hub._devices.values())[0]
+    device = hub._devices["123_grid_30"]
     metric = device.get_metric_from_unique_id("123_grid_30_grid_energy_forward_L1")
     assert metric is not None, "Metric should exist in the device"
     assert metric.value == 42, f"Expected metric value to be 42, got {metric.value}"
@@ -115,35 +140,32 @@ async def test_phase_message(create_mocked_hub):
     assert metric.name == "Grid consumption on L1", f"Expected metric name to be 'Grid consumption on L1', got {metric.name}"
     assert metric.unit_of_measurement == "kWh", f"Expected metric unit_of_measurement to be 'kWh', got {metric.unit_of_measurement}"
 
-
-    # Ensure cleanup happens even if the test fails
-
 @pytest.mark.asyncio
 async def test_placeholder_message(create_mocked_hub):
     """Test that the Hub correctly updates its internal state based on MQTT messages."""
-    hub, connect_task = await create_mocked_hub
+    hub: Hub = create_mocked_hub
 
     # Inject messages after the event is set
-    inject_message(hub, "N/123/system/170/Relay/0/State", "{\"value\": 1}")
-    await finalize_injection(hub, connect_task)
+    inject_message(hub, "N/123/system/0/Relay/0/State", "{\"value\": 1}")
+    await finalize_injection(hub)
 
     # Validate the Hub's state
     assert len(hub._devices) == 1, f"Expected 1 device, got {len(hub._devices)}"
 
     # Validate that the device has the metric we published
-    device = list(hub._devices.values())[0]
-    metric = device.get_metric_from_unique_id("123_system_170_system_relay_0")
+    device =hub._devices["123_system_0"]
+    metric = device.get_metric_from_unique_id("123_system_0_system_relay_0")
     assert metric is not None, "Metric should exist in the device"
     assert metric.value == GenericOnOff.On, f"Expected metric value to be GenericOnOff.On, got {metric.value}"
 
 @pytest.mark.asyncio
 async def test_dynamic_min_max_message(create_mocked_hub):
     """Test that the Hub correctly updates its internal state based on MQTT messages."""
-    hub, connect_task = await create_mocked_hub
+    hub: Hub = create_mocked_hub
 
     # Inject messages after the event is set
     inject_message(hub, "N/123/settings/0/Settings/CGwacs/AcPowerSetPoint", '{"max": 1000000, "min": -1000000, "value": 50}')
-    await finalize_injection(hub, connect_task)
+    await finalize_injection(hub)
 
     # Validate the Hub's state
     assert len(hub._devices) == 1, f"Expected 1 device, got {len(hub._devices)}"
@@ -160,16 +182,16 @@ async def test_dynamic_min_max_message(create_mocked_hub):
 @pytest.mark.asyncio
 async def test_number_message(create_mocked_hub):
     """Test that the Hub correctly updates its internal state based on MQTT messages."""
-    hub, connect_task = await create_mocked_hub
+    hub: Hub = create_mocked_hub
 
     # Inject messages after the event is set
     inject_message(hub, "N/123/evcharger/170/SetCurrent", "{\"value\": 100}")
 
     # Validate the Hub's state
-    assert len(hub._devices) == 1, f"Expected 1 device, got {len(hub._devices)}"
+    assert len(hub._devices) == 2, f"Expected 2 device, got {len(hub._devices)}"
 
     # Validate that the device has the metric we published
-    device = list(hub._devices.values())[0]
+    device = hub._devices["123_evcharger_170"]
     switch = device.get_metric_from_unique_id("123_evcharger_170_evcharger_set_current")
     assert isinstance(switch, Switch), f"Expected switch to be of type Switch, got {type(switch)}"
     assert switch.value == 100, f"Expected switch value to be 100, got {switch.value}"
@@ -181,7 +203,7 @@ async def test_number_message(create_mocked_hub):
         published['value'] = value
         # Call the original publish if needed
         if hasattr(hub.publish, '__wrapped__'):
-            return hub.publish.__wrapped__(topic, value)
+            return hub.publish.__wrapped__(topic, value) # type: ignore
     orig_publish = hub.publish
     hub.publish = mock_publish
 
@@ -196,24 +218,24 @@ async def test_number_message(create_mocked_hub):
     # Restore the original publish method
     hub.publish = orig_publish
 
-    await finalize_injection(hub, connect_task)
+    await finalize_injection(hub)
 
 
 @pytest.mark.asyncio
 async def test_placeholder_adjustable_on(create_mocked_hub):
     """Test that the Hub correctly updates its internal state based on MQTT messages."""
-    hub, connect_task = await create_mocked_hub
+    hub: Hub = create_mocked_hub
 
     # Inject messages after the event is set
     inject_message(hub, "N/123/vebus/170/Ac/ActiveIn/CurrentLimit", "{\"value\": 100}")
     inject_message(hub, "N/123/vebus/170/Ac/ActiveIn/CurrentLimitIsAdjustable", "{\"value\": 1}")
-    await finalize_injection(hub, connect_task)
+    await finalize_injection(hub)
 
     # Validate the Hub's state
-    assert len(hub._devices) == 1, f"Expected 1 device, got {len(hub._devices)}"
+    assert len(hub._devices) == 2, f"Expected 2 device, got {len(hub._devices)}"
 
     # Validate that the device has the metric we published
-    device = list(hub._devices.values())[0]
+    device = hub._devices["123_vebus_170"]
     assert len(device._metrics) == 1, f"Expected 1 metrics, got {len(device._metrics)}"
     metric = device.get_metric_from_unique_id("123_vebus_170_inverter_current_limit")
     assert isinstance(metric, Switch), f"Expected metric to be of type Switch, got {type(metric)}"
@@ -224,18 +246,18 @@ async def test_placeholder_adjustable_on(create_mocked_hub):
 @pytest.mark.asyncio
 async def test_placeholder_adjustable_off(create_mocked_hub):
     """Test that the Hub correctly updates its internal state based on MQTT messages."""
-    hub, connect_task = await create_mocked_hub
+    hub: Hub = create_mocked_hub
 
     # Inject messages after the event is set
     inject_message(hub, "N/123/vebus/170/Ac/ActiveIn/CurrentLimit", "{\"value\": 100}")
     inject_message(hub, "N/123/vebus/170/Ac/ActiveIn/CurrentLimitIsAdjustable", "{\"value\": 0}")
-    await finalize_injection(hub, connect_task)
+    await finalize_injection(hub)
 
     # Validate the Hub's state
-    assert len(hub._devices) == 1, f"Expected 1 device, got {len(hub._devices)}"
+    assert len(hub._devices) == 2, f"Expected 2 device, got {len(hub._devices)}"
 
     # Validate that the device has the metric we published
-    device = list(hub._devices.values())[0]
+    device = hub._devices["123_vebus_170"]
     assert len(device._metrics) == 1, f"Expected 1 metrics, got {len(device._metrics)}"
     metric = device.get_metric_from_unique_id("123_vebus_170_inverter_current_limit")
     assert not isinstance(metric, Switch), f"Expected metric to be of type Metric, got {type(metric)}"
@@ -246,18 +268,15 @@ async def test_placeholder_adjustable_off(create_mocked_hub):
 @pytest.mark.asyncio
 async def test_placeholder_adjustable_on_reverse(create_mocked_hub):
     """Test that the Hub correctly updates its internal state based on MQTT messages."""
-    hub, connect_task = await create_mocked_hub
+    hub: Hub = create_mocked_hub
 
     # Inject messages after the event is set
     inject_message(hub, "N/123/vebus/170/Ac/ActiveIn/CurrentLimitIsAdjustable", "{\"value\": 1}")
     inject_message(hub, "N/123/vebus/170/Ac/ActiveIn/CurrentLimit", "{\"value\": 100}")
-    await finalize_injection(hub, connect_task)
-
-    # Validate the Hub's state
-    assert len(hub._devices) == 1, f"Expected 1 device, got {len(hub._devices)}"
+    await finalize_injection(hub)
 
     # Validate that the device has the metric we published
-    device = list(hub._devices.values())[0]
+    device = hub._devices["123_vebus_170"]
     assert len(device._metrics) == 1, f"Expected 1 metrics, got {len(device._metrics)}"
     metric = device.get_metric_from_unique_id("123_vebus_170_inverter_current_limit")
     assert isinstance(metric, Switch), f"Expected metric to be of type Switch, got {type(metric)}"
@@ -268,18 +287,15 @@ async def test_placeholder_adjustable_on_reverse(create_mocked_hub):
 @pytest.mark.asyncio
 async def test_placeholder_adjustable_off_reverse(create_mocked_hub):
     """Test that the Hub correctly updates its internal state based on MQTT messages."""
-    hub, connect_task = await create_mocked_hub
+    hub: Hub = create_mocked_hub
 
     # Inject messages after the event is set
     inject_message(hub, "N/123/vebus/170/Ac/ActiveIn/CurrentLimitIsAdjustable", "{\"value\": 0}")
     inject_message(hub, "N/123/vebus/170/Ac/ActiveIn/CurrentLimit", "{\"value\": 100}")
-    await finalize_injection(hub, connect_task)
-
-    # Validate the Hub's state
-    assert len(hub._devices) == 1, f"Expected 1 device, got {len(hub._devices)}"
+    await finalize_injection(hub)
 
     # Validate that the device has the metric we published
-    device = list(hub._devices.values())[0]
+    device = hub._devices["123_vebus_170"]
     assert len(device._metrics) == 1, f"Expected 1 metrics, got {len(device._metrics)}"
     metric = device.get_metric_from_unique_id("123_vebus_170_inverter_current_limit")
     assert not isinstance(metric, Switch), f"Expected metric to be of type Metric, got {type(metric)}"
@@ -291,18 +307,15 @@ async def test_placeholder_adjustable_off_reverse(create_mocked_hub):
 @pytest.mark.asyncio
 async def test_today_message(create_mocked_hub):
     """Test that the Hub correctly updates its internal state based on MQTT messages."""
-    hub, connect_task = await create_mocked_hub
+    hub: Hub = create_mocked_hub
 
     # Inject messages after the event is set
     inject_message(hub, "N/123/solarcharger/290/History/Daily/0/MaxPower", "{\"value\": 1}")
     inject_message(hub, "N/123/solarcharger/290/History/Daily/1/MaxPower", "{\"value\": 2}")
-    await finalize_injection(hub, connect_task)
-
-    # Validate the Hub's state
-    assert len(hub._devices) == 1, f"Expected 1 device, got {len(hub._devices)}"
+    await finalize_injection(hub)
 
     # Validate that the device has the metric we published
-    device = list(hub._devices.values())[0]
+    device = hub._devices["123_solarcharger_290"]
     assert len(device._metrics) == 2, f"Expected 2 metrics, got {len(device._metrics)}"
 
     metric = device.get_metric_from_unique_id("123_solarcharger_290_solarcharger_max_power_today")
@@ -328,19 +341,178 @@ def test_expend_topics():
 @pytest.mark.asyncio
 async def test_expend_message(create_mocked_hub):
     """Test that the Hub correctly updates its internal state based on MQTT messages."""
-    hub, connect_task = await create_mocked_hub
+    hub: Hub = create_mocked_hub
 
     # Inject messages after the event is set
     inject_message(hub, "N/123/switch/170/SwitchableOutput/output_2/State", "{\"value\": 1}")
-    await finalize_injection(hub, connect_task)
-
-    # Validate the Hub's state
-    assert len(hub._devices) == 1, f"Expected 1 device, got {len(hub._devices)}"
+    await finalize_injection(hub)
 
     # Validate that the device has the metric we published
-    device = list(hub._devices.values())[0]
+    device = hub._devices["123_switch_170"]
     metric = device.get_metric_from_unique_id("123_switch_170_switch_2_state")
     assert metric is not None, "Metric should exist in the device"
     assert metric.generic_short_id == "switch_{output}_state"
     assert metric.key_values["output"] == "2"
     assert metric.value == GenericOnOff.On, f"Expected metric value to be GenericOnOff.On, got {metric.value}"
+
+@pytest.mark.asyncio
+async def test_expend_message_2(create_mocked_hub):
+    """Test that the Hub correctly updates its internal state based on MQTT messages."""
+    hub: Hub = create_mocked_hub
+
+    # Inject messages after the event is set
+    inject_message(hub, "N/123/battery/170/Voltages/Cell3", "{\"value\": 3.331}")
+    await finalize_injection(hub)
+
+    # Validate that the device has the metric we published
+    device = hub._devices["123_battery_170"]
+    metric = device.get_metric_from_unique_id("123_battery_170_battery_cell_3_voltage")
+    assert metric is not None, "Metric should exist in the device"
+    assert metric.generic_short_id == "battery_cell_{cell_id}_voltage"
+    assert metric.key_values["cell_id"] == "3"
+    assert metric.value == 3.331, f"Expected metric value to be 3.331, got {metric.value}"
+
+@pytest.mark.asyncio
+async def test_same_message_events(create_mocked_hub):
+    """Test that the Hub correctly updates its internal state based on MQTT messages."""
+    hub: Hub = create_mocked_hub
+
+    # Inject messages after the event is set
+    inject_message(hub, "N/123/grid/30/Ac/L1/Energy/Forward", "{\"value\": 42}")
+    await finalize_injection(hub, False)
+
+    # Validate that the device has the metric we published
+    device = hub._devices["123_grid_30"]
+    metric = device.get_metric_from_unique_id("123_grid_30_grid_energy_forward_L1")
+    assert metric is not None, "Metric should exist in the device"
+    assert metric.value == 42, f"Expected metric value to be 42, got {metric.value}"
+    metric.on_update = MagicMock()
+
+    # Inject the same message again
+    inject_message(hub, "N/123/grid/30/Ac/L1/Energy/Forward", "{\"value\": 42}")
+    await asyncio.sleep(0.01)  # Allow event loop to process any scheduled callbacks
+    assert metric.on_update.call_count == 0, "on_update should not be called for the same value"
+    inject_message(hub, "N/123/grid/30/Ac/L1/Energy/Forward", "{\"value\": 43}")
+    await asyncio.sleep(0.01)  # Allow event loop to process any scheduled callbacks
+    assert metric.on_update.call_count == 1, "on_update should be called for the new value"
+
+    await hub.disconnect()
+
+@pytest.mark.asyncio
+async def test_existing_installation_id(create_mocked_hub_with_installation_id):
+    """Test that the Hub correctly updates its internal state based on MQTT messages."""
+    hub: Hub = create_mocked_hub_with_installation_id
+
+    # Inject messages after the event is set
+    inject_message(hub, "N/123/switch/170/SwitchableOutput/output_2/State", "{\"value\": 1}")
+    await finalize_injection(hub)
+
+    # Validate that the device has the metric we published
+    device = hub._devices["123_switch_170"]
+    metric = device.get_metric_from_unique_id("123_switch_170_switch_2_state")
+    assert metric is not None, "Metric should exist in the device"
+    assert metric.generic_short_id == "switch_{output}_state"
+    assert metric.key_values["output"] == "2"
+    assert metric.value == GenericOnOff.On, f"Expected metric value to be GenericOnOff.On, got {metric.value}"
+
+
+@pytest.mark.asyncio
+async def test_multiple_hubs(create_mocked_hub_with_installation_id, create_mocked_hub):
+    """Test that the Hub correctly updates its internal state based on MQTT messages."""
+    hub1: Hub = create_mocked_hub
+
+    # Inject messages after the event is set
+    inject_message(hub1, "N/123/switch/170/SwitchableOutput/output_2/State", "{\"value\": 1}")
+    await finalize_injection(hub1, disconnect=False)
+
+    # Validate that the device has the metric we published
+    device1 = hub1._devices["123_switch_170"]
+    metric1 = device1.get_metric_from_unique_id("123_switch_170_switch_2_state")
+    assert metric1 is not None, "Metric should exist in the device"
+    assert metric1.generic_short_id == "switch_{output}_state"
+    assert metric1.key_values["output"] == "2"
+    assert metric1.value == GenericOnOff.On, f"Expected metric value to be GenericOnOff.On, got {metric1.value}"
+
+    hub2: Hub = create_mocked_hub_with_installation_id
+    # Inject messages after the event is set
+    inject_message(hub2, "N/123/switch/170/SwitchableOutput/output_2/State", "{\"value\": 0}")
+    await finalize_injection(hub1, disconnect=False)
+
+    # Validate the Hub's state
+    assert len(hub2._devices) == 1, f"Expected 1 device, got {len(hub1._devices)}"
+
+    # Validate that the device has the metric we published
+    device2 = hub2._devices["123_switch_170"]
+    metric2 = device2.get_metric_from_unique_id("123_switch_170_switch_2_state")
+    assert metric2 is not None, "Metric should exist in the device"
+    assert metric2.generic_short_id == "switch_{output}_state"
+    assert metric2.key_values["output"] == "2"
+    assert metric2.value == GenericOnOff.Off, f"Expected metric value to be GenericOnOff.Off, got {metric2.value}"
+
+    await hub2.disconnect()
+    await hub1.disconnect()
+
+@pytest.mark.asyncio
+async def test_float_precision(create_mocked_hub):
+    """Test that the Hub correctly updates its internal state based on MQTT messages."""
+    hub: Hub = create_mocked_hub
+
+    # Inject messages after the event is set
+    inject_message(hub, "N/123/system/170/Dc/System/Power", "{\"value\": 1.1234}")
+    await finalize_injection(hub)
+
+    # Validate that the device has the metric we published
+    device = hub._devices["123_system_170"]
+    metric = device.get_metric_from_unique_id("123_system_170_system_dc_consumption")
+    assert metric is not None, "Metric should exist in the device"
+    assert metric.value == 1.1, f"Expected metric value to be 1.1, got {metric.value}"
+
+@pytest.mark.asyncio
+async def test_float_precision_none(create_mocked_hub):
+    """Test that the Hub correctly updates its internal state based on MQTT messages."""
+    hub: Hub = create_mocked_hub
+
+    # Inject messages after the event is set
+    inject_message(hub, "N/123/gps/170/Position/Latitude", "{\"value\": 1.0123456789}")
+    await finalize_injection(hub)
+
+    # Validate that the device has the metric we published
+    device = hub._devices["123_gps_170"]
+    metric = device.get_metric_from_unique_id("123_gps_170_gps_latitude")
+    assert metric is not None, "Metric should exist in the device"
+    assert metric.value == 1.0123456789, f"Expected metric value to be 1.0123456789, got {metric.value}"
+
+@pytest.mark.asyncio
+async def test_new_metric(create_mocked_hub):
+    """Test that the Hub correctly triggers the on_new_metric callback."""
+    hub: Hub = create_mocked_hub
+
+    # Mock the on_new_metric callback
+    def on_new_metric_mock(hub, device, metric):
+        logger.debug(f"New metric added: Hub={hub}, Device={device}, Metric={repr(metric)}")
+    hub.on_new_metric = MagicMock(side_effect=on_new_metric_mock)
+
+    # Inject messages after the event is set
+    inject_message(hub, "N/123/system/170/Dc/System/Power", "{\"value\": 1.1234}")
+    await finalize_injection(hub, disconnect=False)
+
+    # Wait for the callback to be triggered
+    await asyncio.sleep(0.1)  # Allow event loop to process the callback
+
+    # Validate that the on_new_metric callback was called
+    hub.on_new_metric.assert_called_once()
+
+    # Check that we got the callback only once
+    await hub._keepalive()
+    # Wait for the callback to be triggered
+    await asyncio.sleep(0.1)  # Allow event loop to process the callback
+    hub.on_new_metric.assert_called_once()
+
+    # Validate that the device has the metric we published
+    device = hub._devices["123_system_170"]
+    metric = device.get_metric_from_unique_id("123_system_170_system_dc_consumption")
+    assert metric is not None, "Metric should exist in the device"
+    assert metric.value == 1.1, f"Expected metric value to be 1.1, got {metric.value}"
+
+
+    await hub.disconnect()
