@@ -5,7 +5,7 @@ Maps all the MQTT topics to either attributes or metrics.
 from typing import List
 from .constants import MetricKind, MetricNature, MetricType, ValueType, RangeType
 from .data_classes import TopicDescriptor
-from ._victron_enums import DeviceType, ESSMode, InverterMode, GenericOnOff, EvChargerMode, InverterOverloadAlarmEnum, InverterState, MultiState, SolarChargerState, TemperatureStatus, TemperatureType
+from ._victron_enums import DESSReactiveStrategy, DESSStrategy, DeviceType, ESSMode, FluidType, InverterMode, GenericOnOff, EvChargerMode, InverterOverloadAlarmEnum, InverterState, MultiState, SolarChargerState, TemperatureStatus, TemperatureType, DESSErrorCode, DESSRestrictions
 
 topics: List[TopicDescriptor] = [
     # generic device attributes
@@ -268,7 +268,7 @@ topics: List[TopicDescriptor] = [
         metric_nature=MetricNature.INSTANTANEOUS,
         device_type=DeviceType.SOLAR_CHARGER,
         value_type=ValueType.FLOAT,
-        precision=2,
+        precision=3,
     ),
     TopicDescriptor(
         topic="N/+/solarcharger/+/Dc/0/Current",
@@ -431,6 +431,15 @@ topics: List[TopicDescriptor] = [
         value_type=ValueType.ENUM,
         enum=GenericOnOff,
     ),
+    TopicDescriptor(
+        topic="N/+/solarcharger/+/MppOperationMode",
+        message_type=MetricKind.SENSOR,
+        short_id="solarcharger_mppt_operation_mode",
+        name="Solar Charger MPPT Operation Mode",
+        device_type=DeviceType.SOLAR_CHARGER,
+        value_type=ValueType.ENUM,
+        enum=MultiState,
+    ),
     TopicDescriptor(
         topic="N/+/solarcharger/+/Relay/0/State",
         message_type=MetricKind.SWITCH,
@@ -451,7 +460,7 @@ topics: List[TopicDescriptor] = [
         metric_nature=MetricNature.INSTANTANEOUS,
         device_type=DeviceType.BATTERY,
         value_type=ValueType.FLOAT,
-        precision=1,
+        precision=3,
     ),
     TopicDescriptor(
         topic="N/+/battery/+/Dc/0/Current",
@@ -601,19 +610,38 @@ topics: List[TopicDescriptor] = [
         short_id="charge_mode",
         name="Battery charge mode",
         device_type=DeviceType.BATTERY,
-        unit_of_measurement=None,
-        metric_type=MetricType.NONE,
-        metric_nature=MetricNature.NONE,
         value_type=ValueType.STRING,
     ),
+    TopicDescriptor(
+        topic="N/+/battery/+/ConsumedAmphours",
+        message_type=MetricKind.SENSOR,
+        short_id="battery_consumed_amphours",
+        name="Battery consumed amp-hours",
+        device_type=DeviceType.BATTERY,
+        unit_of_measurement="Ah",
+        metric_type=MetricType.ELECTRIC_STORAGE_CAPACITY,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        value_type=ValueType.FLOAT,
+        precision=1,
+    ),
+    TopicDescriptor(
+        topic="N/+/battery/+/Voltages/Cell{cell_id(1-4)}",
+        message_type=MetricKind.SENSOR,
+        short_id="battery_cell_{cell_id}_voltage",
+        name="Battery cell {cell_id} voltage",
+        unit_of_measurement="V",
+        metric_type=MetricType.VOLTAGE,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        device_type=DeviceType.BATTERY,
+        value_type=ValueType.FLOAT,
+        precision=3,
+    ),
     # inverter
     TopicDescriptor(
         topic="N/+/vebus/+/Mode",
         message_type=MetricKind.SELECT,
         short_id="inverter_mode",
         name="Inverter mode",
-        metric_type=MetricType.NONE,
-        metric_nature=MetricNature.NONE,
         device_type=DeviceType.INVERTER,
         value_type=ValueType.ENUM,
         enum=InverterMode,
@@ -623,12 +651,22 @@ topics: List[TopicDescriptor] = [
         message_type=MetricKind.SENSOR,
         short_id="inverter_state",
         name="Inverter state",
-        metric_type=MetricType.NONE,
-        metric_nature=MetricNature.NONE,
         device_type=DeviceType.INVERTER,
         value_type=ValueType.ENUM,
         enum=InverterState,
     ),
+    TopicDescriptor(
+        topic="N/+/vebus/+/Ac/ActiveIn/{phase}/V",
+        message_type=MetricKind.SENSOR,
+        short_id="inverter_input_voltage_{phase}",
+        name="Inverter input voltage {phase}",
+        unit_of_measurement="V",
+        metric_type=MetricType.VOLTAGE,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        device_type=DeviceType.INVERTER,
+        value_type=ValueType.FLOAT,
+        precision=1,
+    ),
     TopicDescriptor(
         topic="N/+/vebus/+/Ac/ActiveIn/{phase}/P",
         message_type=MetricKind.SENSOR,
@@ -777,17 +815,178 @@ topics: List[TopicDescriptor] = [
         value_type=ValueType.FLOAT,
         precision=1,
     ),
+    TopicDescriptor(
+        topic="N/+/system/+/Dc/Battery/Soc",
+        message_type=MetricKind.SENSOR,
+        short_id="system_dc_battery_soc",
+        name="DC Battery Charge",
+        unit_of_measurement="%",
+        metric_type=MetricType.PERCENTAGE,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.FLOAT,
+        precision=1,
+    ),
+    TopicDescriptor(
+        topic="N/+/system/+/Dc/Battery/Power",
+        message_type=MetricKind.SENSOR,
+        short_id="system_dc_battery_power",
+        name="DC Battery Power",
+        unit_of_measurement="W",
+        metric_type=MetricType.POWER,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.FLOAT,
+        precision=1,
+    ),
+    TopicDescriptor(
+        topic="N/+/system/+/Dc/Battery/Voltage",
+        message_type=MetricKind.SENSOR,
+        short_id="system_dc_battery_voltage",
+        name="DC Battery Voltage",
+        unit_of_measurement="V",
+        metric_type=MetricType.VOLTAGE,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.FLOAT,
+        precision=3,
+    ),
+    TopicDescriptor(
+        topic="N/+/system/+/Dc/Battery/Current",
+        message_type=MetricKind.SENSOR,
+        short_id="system_dc_battery_current",
+        name="DC Battery Current",
+        unit_of_measurement="A",
+        metric_type=MetricType.CURRENT,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.FLOAT,
+        precision=1
+    ),
     TopicDescriptor(
         topic="N/+/system/+/Relay/{relay}/State",
         message_type=MetricKind.SWITCH,
         short_id="system_relay_{relay}",
         name="Relay {relay} state",
-        metric_type=MetricType.NONE,
-        metric_nature=MetricNature.NONE,
         device_type=DeviceType.SYSTEM,
         value_type=ValueType.ENUM,
         enum=GenericOnOff,
     ),
+    TopicDescriptor(
+        topic="N/+/system/+/DynamicEss/Available",
+        message_type=MetricKind.BINARY_SENSOR,
+        short_id="system_dynamicess_available",
+        name="Dynamic ESS Available",
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.ENUM,
+        enum=GenericOnOff
+    ),
+    # there is at least one mode for DESS that is controlled by Node-Red (value=4)
+    # it may show up here - as well as: GREEN Mode (1?)an Trade Mode (2?)
+    TopicDescriptor(
+        topic="N/+/system/+/DynamicEss/Active",
+        message_type=MetricKind.SWITCH,
+        short_id="system_dynamicess_active",
+        name="Dynamic ESS Active",
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.ENUM,
+        enum=GenericOnOff,
+    ),
+    TopicDescriptor(
+        topic="N/+/system/+/DynamicEss/AllowGridFeedIn",
+        message_type=MetricKind.BINARY_SENSOR,
+        short_id="system_dynamicess_allow_gridfeedin",
+        name="Dynamic ESS Allow Grid Feed In",
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.ENUM,
+        enum=GenericOnOff,
+    ),
+    TopicDescriptor(
+        topic="N/+/system/+/DynamicEss/AvailableOverhead",
+        message_type=MetricKind.SENSOR,
+        short_id="system_dynamicess_available_overhead",
+        name="Dynamic ESS Available Overhead",
+        device_type=DeviceType.SYSTEM,
+        unit_of_measurement="W",
+        metric_type=MetricType.POWER,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        value_type=ValueType.FLOAT,
+        precision=1,
+    ),
+    TopicDescriptor(
+        topic="N/+/system/+/DynamicEss/ErrorCode",
+        message_type=MetricKind.SWITCH,
+        short_id="system_dynamicess_error",
+        name="Dynamic ESS Error",
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.ENUM,
+        enum=DESSErrorCode,
+    ),
+    TopicDescriptor(
+        topic="N/+/system/+/DynamicEss/NumberOfSchedules",
+        message_type=MetricKind.SENSOR,
+        short_id="system_dynamicess_schedule_count",
+        name="Dynamic ESS Number Of Schedules",
+        device_type=DeviceType.SYSTEM,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        value_type=ValueType.INT,
+    ),
+    TopicDescriptor(
+        topic="N/+/system/+/DynamicEss/TargetSoc",
+        message_type=MetricKind.SENSOR,
+        short_id="system_dynamicess_target_soc",
+        name="Dynamic ESS Target SOC",
+        unit_of_measurement="%",
+        metric_type=MetricType.PERCENTAGE,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.FLOAT,
+        precision=0,
+    ),
+    TopicDescriptor(
+        topic="N/+/system/+/DynamicEss/MinimumSoc",
+        message_type=MetricKind.SENSOR,
+        short_id="system_dynamicess_minimum_soc",
+        name="Dynamic ESS Minimum SOC",
+        unit_of_measurement="%",
+        metric_type=MetricType.PERCENTAGE,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.FLOAT,
+        precision=0,
+    ),
+    # N/+/system/+/DynamicEss/Capabilities       - not documented - in my case always 31 (bitmask?)
+    # N/+/system/+/DynamicEss/ChargeRate         - not documented - in my case alway 0
+    # N/+/system/+/DynamicEss/Flags              - not documented - in my case always 0
+    # N/+/system/+/DynamicEss/LastScheduledStart - unix timestamp - needs conversion to time
+    # N/+/system/+/DynamicEss/LastScheduledEnd   - unix timestamp - needs conversion to time
+    TopicDescriptor(
+        topic="N/+/system/+/DynamicEss/Restrictions",
+        message_type=MetricKind.SENSOR,
+        short_id="system_dynamicess_restrictions",
+        name="Dynamic ESS Restrictions",
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.ENUM,
+        enum=DESSRestrictions,
+    ),
+    TopicDescriptor(
+        topic="N/+/system/+/DynamicEss/Strategy",
+        message_type=MetricKind.SENSOR,
+        short_id="system_dynamicess_strategy",
+        name="Dynamic ESS Strategy",
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.ENUM,
+        enum=DESSStrategy,
+    ),
+    TopicDescriptor(
+        topic="N/+/system/+/DynamicEss/ReactiveStrategy",
+        message_type=MetricKind.SENSOR,
+        short_id="system_dynamicess_reactive_strategy",
+        name="Dynamic ESS Reactive Strategy",
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.ENUM,
+        enum=DESSReactiveStrategy,
+    ),
     TopicDescriptor(
         topic="N/+/system/+/Ac/Genset/{phase}/Power",
         message_type=MetricKind.SENSOR,
@@ -1051,6 +1250,15 @@ topics: List[TopicDescriptor] = [
         value_type=ValueType.FLOAT,
         precision=1,
     ),
+    TopicDescriptor(
+        topic="N/+/tank/+/FluidType",
+        message_type=MetricKind.SENSOR,
+        short_id="tank_fluid_type",
+        name="Fluid Type",
+        device_type=DeviceType.TANK,
+        value_type=ValueType.ENUM,
+        enum=FluidType,
+    ),
     #multirssolar
     #acin and acout
     TopicDescriptor(
@@ -1255,7 +1463,7 @@ topics: List[TopicDescriptor] = [
         metric_nature=MetricNature.INSTANTANEOUS,
         device_type=DeviceType.ALTERNATOR,
         value_type=ValueType.FLOAT,
-        precision=2,
+        precision=3,
     ),
     TopicDescriptor(
         topic="N/+/alternator/+/Dc/0/Current",
@@ -1332,6 +1540,51 @@ topics: List[TopicDescriptor] = [
         min=-10000,  # Dynamic range, depends on device model,
         max=10000,  # Dynamic range, depends on device model
     ),
+    TopicDescriptor(
+        topic="N/+/settings/+/Settings/CGwacs/OvervoltageFeedIn",
+        message_type=MetricKind.SWITCH,
+        short_id="system_settings_overvoltage_feedin",
+        name="PV DC Overvoltage FeedIn",
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.ENUM,
+        enum=GenericOnOff,
+    ),
+    TopicDescriptor(
+        topic="N/+/settings/+/Settings/CGwacs/BatteryLife/MinimumSocLimit",
+        message_type=MetricKind.NUMBER,
+        short_id="system_ess_min_soc_limit",
+        name="ESS min SOC limit",
+        unit_of_measurement="%",
+        metric_type=MetricType.PERCENTAGE,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.INT,
+        min_max_range=RangeType.DYNAMIC,
+    ),
+    TopicDescriptor(
+        topic="N/+/settings/+/Settings/SystemSetup/MaxChargeCurrent",
+        message_type=MetricKind.NUMBER,
+        short_id="system_ess_max_charge_current",
+        name="ESS max charge current",
+        unit_of_measurement="A",
+        metric_type=MetricType.CURRENT,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.INT,
+        min_max_range=RangeType.DYNAMIC,
+    ),
+    TopicDescriptor(
+        topic="N/+/settings/+/Settings/SystemSetup/MaxChargeVoltage",
+        message_type=MetricKind.NUMBER,
+        short_id="system_ess_max_charge_voltage",
+        name="ESS max charge voltage",
+        unit_of_measurement="V",
+        metric_type=MetricType.VOLTAGE,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.INT,
+        min_max_range=RangeType.DYNAMIC,
+    ),
     # DC Load
     TopicDescriptor(
         topic="N/+/dcload/+/Dc/0/Voltage",
@@ -1343,7 +1596,7 @@ topics: List[TopicDescriptor] = [
         metric_nature=MetricNature.INSTANTANEOUS,
         device_type=DeviceType.DC_LOAD,
         value_type=ValueType.FLOAT,
-        precision=2
+        precision=3
     ),
     TopicDescriptor(
         topic="N/+/dcload/+/Dc/0/Current",
@@ -1392,12 +1645,13 @@ topics: List[TopicDescriptor] = [
     # GPS
     TopicDescriptor(
         topic="N/+/gps/+/Altitude",
-        message_type=MetricKind.NUMBER,
+        message_type=MetricKind.SENSOR,
         short_id="gps_altitude",
         name="GPS Altitude",
 		metric_nature=MetricNature.INSTANTANEOUS,
         device_type=DeviceType.GPS,
         value_type=ValueType.FLOAT,
+        precision=None # Use full precision of GPS device
     ),
     TopicDescriptor(
         topic="N/+/gps/+/Connected",
@@ -1410,7 +1664,7 @@ topics: List[TopicDescriptor] = [
     ),
     TopicDescriptor(
         topic="N/+/gps/+/Course",
-        message_type=MetricKind.NUMBER,
+        message_type=MetricKind.SENSOR,
         short_id="gps_course",
         name="GPS Course",
 		metric_nature=MetricNature.INSTANTANEOUS,
@@ -1429,7 +1683,7 @@ topics: List[TopicDescriptor] = [
     ),
     TopicDescriptor(
         topic="N/+/gps/+/Speed",
-        message_type=MetricKind.NUMBER,
+        message_type=MetricKind.SENSOR,
         short_id="gps_speed",
         name="GPS Speed",
 		metric_nature=MetricNature.INSTANTANEOUS,
@@ -1439,7 +1693,7 @@ topics: List[TopicDescriptor] = [
     ),
     TopicDescriptor(
         topic="N/+/gps/+/NrOfSatellites",
-        message_type=MetricKind.NUMBER,
+        message_type=MetricKind.SENSOR,
         short_id="gps_nrofsatellites",
         name="GPS Nr Of Satellites",
 		metric_nature=MetricNature.INSTANTANEOUS,
@@ -1448,22 +1702,36 @@ topics: List[TopicDescriptor] = [
     ),
     TopicDescriptor(
         topic="N/+/gps/+/Position/Latitude",
-        message_type=MetricKind.NUMBER,
+        message_type=MetricKind.SENSOR,
         short_id="gps_latitude",
         name="GPS Latitude",
 		metric_nature=MetricNature.INSTANTANEOUS,
 		metric_type=MetricType.LOCATION,
         device_type=DeviceType.GPS,
         value_type=ValueType.FLOAT,
+        precision=None # Use full precision of GPS device
     ),
     TopicDescriptor(
         topic="N/+/gps/+/Position/Longitude",
-        message_type=MetricKind.NUMBER,
+        message_type=MetricKind.SENSOR,
         short_id="gps_longitude",
         name="GPS Longitude",
 		metric_nature=MetricNature.INSTANTANEOUS,
 		metric_type=MetricType.LOCATION,
         device_type=DeviceType.GPS,
         value_type=ValueType.FLOAT,
+        precision=None # Use full precision of GPS device
+    ),
+    TopicDescriptor(
+        topic="N/+/settings/+/Settings/CGwacs/MaxDischargePower",
+        message_type=MetricKind.NUMBER,
+        short_id="system_ess_max_inverter_power_limit",
+        name="ESS max inverter power limit",
+        unit_of_measurement="W",
+        metric_type=MetricType.POWER,
+        metric_nature=MetricNature.INSTANTANEOUS,
+        device_type=DeviceType.SYSTEM,
+        value_type=ValueType.INT,
+        min_max_range=RangeType.DYNAMIC,
     ),
 ]
