@@ -3,9 +3,11 @@
 import asyncio
 import json
 import logging
+import random
 import ssl
 import re
-from typing import Any, Optional
+import string
+from typing import Any, Callable, Optional, Tuple
 
 import paho.mqtt.client as mqtt
 from paho.mqtt.client import Client as MQTTClient, PayloadType
@@ -20,6 +22,21 @@ from .device import Device
 from .metric import Metric
 
 _LOGGER = logging.getLogger(__name__)
+CONNECT_MAX_FAILED_ATTEMPTS = 3
+
+# Modify the logger to include instance_id without changing the tracing level
+# class InstanceIDFilter(logging.Filter):
+#     def __init__(self, instance_id):
+#         super().__init__()
+#         self.instance_id = instance_id
+
+#     def filter(self, record):
+#         record.instance_id = self.instance_id
+#         return True
+
+# Update the log format to include instance_id with a default value if not present
+# for handler in logging.getLogger().handlers:
+#     handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - [ID: %(instance_id)s] - %(message)s', defaults={'instance_id': 'N/A'}))
 
 # class TracedTask(asyncio.Task):
 #     def __init__(self, coro, *, loop=None, name=None):
@@ -35,6 +52,10 @@ _LOGGER = logging.getLogger(__name__)
 #         else:
 #             print(f"[TASK DONE] {self.get_name()} - Result: {result}")
 
+running_client_id=0
+
+CallbackOnNewMetric = Callable[["Hub", Device, Metric], None]
+
 class Hub:
     """Class to communicate with the Venus OS hub."""
 
@@ -49,8 +70,19 @@ class Hub:
         model_name: str | None = None,
         serial: str | None = "noserial",
         topic_prefix: str | None = None,
+        topic_log_info: str | None = None
     ) -> None:
         """Initialize."""
+        global running_client_id
+        self._instance_id = running_client_id
+        running_client_id += 1
+
+        # Add the instance_id filter to the logger only if it doesn't already exist
+        # self.logger = logging.getLogger(__name__)
+        # self.logger.addFilter(InstanceIDFilter(self._instance_id))
+        # if logger_level is not None:
+        #     self.logger.setLevel(logger_level)
+
         # Parameter validation
         if not isinstance(host, str) or not host:
             raise ValueError("host must be a non-empty string")
@@ -84,14 +116,21 @@ class Hub:
         self.port = port
         self._installation_id = installation_id
         self._topic_prefix = topic_prefix
-        self.logger = logging.getLogger(__name__)
         self._devices: dict[str, Device] = {}
         self._first_refresh_event: asyncio.Event = asyncio.Event()
         self._installation_id_event: asyncio.Event = asyncio.Event()
         self._snapshot = {}
-        self._keep_alive_task = None
+        self._keepalive_task = None
         self._connected_event = asyncio.Event()
-        self._connected_failed = False
+        self._connected_failed_attempts = 0
+        self._on_new_metric: CallbackOnNewMetric | None = None
+        self._topic_log_info = topic_log_info
+        # The client ID is generated using a random string and the instance ID. It has to be unique between all clients connected to the same mqtt server. If not, they may reset each other connection.
+        random_string = ''.join(random.choices(string.ascii_letters + string.digits, k=8))
+        self._client_id = f"victron_mqtt-{random_string}-{self._instance_id}"
+        self._keepalive_counter = 0
+        self._new_metrics: list[Tuple[Device, Metric]] = []
+
         # Replace all {placeholder} patterns with + for MQTT wildcards
         expanded_topics = Hub.expand_topic_list(topics)
         def merge_is_adjustable_suffix(desc: TopicDescriptor) -> str:
@@ -119,13 +158,14 @@ class Hub:
         subscription_list1 = [Hub._remove_placeholders(topic.topic) for topic in expanded_topics]
         subscription_list2 = [Hub._remove_placeholders(merge_is_adjustable_suffix(desc)) for desc in expanded_topics if desc.is_adjustable_suffix]
         self._subscription_list = subscription_list1 + subscription_list2
-        self._client = MQTTClient(callback_api_version=CallbackAPIVersion.VERSION2)
-        _LOGGER.info("Hub initialized")
+        self._client = MQTTClient(callback_api_version=CallbackAPIVersion.VERSION2, client_id=self._client_id)
+        _LOGGER.info("Hub initialized. Client ID: %s", self._client_id)
 
     async def connect(self) -> None:
         """Connect to the hub."""
         _LOGGER.info("Connecting to MQTT broker at %s:%d", self.host, self.port)
-        
+        assert self._client is not None
+
         if self.username is not None:
             _LOGGER.info("Setting auth credentials for user: %s", self.username)
             self._client.username_pw_set(self.username, self.password)
@@ -142,9 +182,9 @@ class Hub:
         self._client.on_message = self._on_message
         self._client.on_connect_fail = self.on_connect_fail
         #self._client.on_log = self._on_log
-        self._connected_failed = False
+        self._connected_failed_attempts = 0
         self._loop = asyncio.get_event_loop()
-        #self._loop.set_task_factory(lambda loop, coro: TracedTask(coro, loop=loop))
+        #self._loop.set_task_factory(lambda loop, coro: TracedTask(coro, loop=loop, name=name))
 
         try:
             _LOGGER.info("Starting paho mqtt")
@@ -153,7 +193,7 @@ class Hub:
             self._client.connect_async(self.host, self.port)
             _LOGGER.info("Waiting for connection event")
             await self._wait_for_connect()
-            if self._connected_failed:
+            if self._connected_failed_attempts >= CONNECT_MAX_FAILED_ATTEMPTS:
                 _LOGGER.error("Failed to connect to MQTT broker")
                 raise CannotConnectError("Failed to connect to MQTT broker")
             _LOGGER.info("Successfully connected to MQTT broker at %s:%d", self.host, self.port)
@@ -161,9 +201,7 @@ class Hub:
                 _LOGGER.info("No installation ID provided, attempting to read from device")
                 self._installation_id = await self._read_installation_id()
             self._start_keep_alive_loop()
-            _LOGGER.info("Waiting for first refresh")
-            await self._wait_for_first_refresh()
-            _LOGGER.info("Devices and metrics initialized. Found %d devices", len(self._devices))
+            _LOGGER.info("Connected. Installation ID: %s", self._installation_id)
         except Exception as exc:
             _LOGGER.error("Failed to connect to MQTT broker: %s", exc, exc_info=True)
             raise CannotConnectError(f"Failed to connect to MQTT broker: {exc}") from exc
@@ -205,40 +243,59 @@ class Hub:
     def _on_message_internal(self, client: MQTTClient, userdata: Any, message: mqtt.MQTTMessage) -> None:
         """Process MQTT message asynchronously."""
         topic = message.topic
-        payload = message.payload
-        _LOGGER.debug("Message received: topic=%s, payload=%s", topic, payload)
-        
+        payload = message.payload.decode()
+
+        # Determine log level based on the substring
+        is_info_level = self._topic_log_info and self._topic_log_info in topic
+        log_debug = _LOGGER.info if is_info_level else _LOGGER.debug
+
+        log_debug("Message received: topic=%s, payload=%s", topic, payload)
+
         # Remove topic prefix before processing
         topic = self._remove_topic_prefix(topic)
-        
+
         if "full_publish_completed" in topic:
-            _LOGGER.info("Full publish completed, unsubscribing from notification")
-            if self._client is not None and self._client.is_connected():
-                self._unsubscribe("N/+/full_publish_completed")
-            self._loop.call_soon_threadsafe(self._first_refresh_event.set)
-            return
+            # Check if this is our full publish completed message
+            echo = self.get_keepalive_echo(payload)
+            # Check if it matches our client ID so we got full cycle or refresh
+            if echo.startswith(self._client_id):
+                log_debug("Full publish completed: %s", echo)
+                # We are sending the new metrics now as we can be sure that the metric handled all the attribute topics and now ready.
+                for device, new_metric in self._new_metrics:
+                    try:
+                        if callable(self._on_new_metric):
+                            if self._loop.is_running():
+                                # If the event loop is running, schedule the callback
+                                self._loop.call_soon_threadsafe(self._on_new_metric, self, device, new_metric)
+                    except Exception as exc:
+                        _LOGGER.error("Error calling _on_new_metric callback %s", exc, exc_info=True)
+                self._new_metrics.clear()
+                self._loop.call_soon_threadsafe(self._first_refresh_event.set)
+                return
+            else:
+                log_debug("Not our echo: %s", echo)
 
-        if not self._installation_id_event.is_set():
-            self._handle_installation_id_message(topic, payload)
-            return
+        if self._installation_id is None and not self._installation_id_event.is_set():
+            self._handle_installation_id_message(topic, log_debug)
 
-        self._handle_normal_message(topic, payload)
+        self._handle_normal_message(topic, payload, log_debug)
 
-    def _handle_installation_id_message(self, topic: str, payload: bytes) -> None:
+    def _handle_installation_id_message(self, topic: str, log_debug) -> None:
         """Handle installation ID message."""
-        topic_parts = topic.split("/")
-        if len(topic_parts) == 5 and topic_parts[2:5] == ["system", "0", "Serial"]:
-            payload_json = json.loads(payload.decode())
-            self._installation_id = payload_json.get("value")
-            _LOGGER.info("Installation ID received: %s. Original topic: %s", self._installation_id, topic)
-            if self._installation_id_event:
-                 self._loop.call_soon_threadsafe(self._installation_id_event.set)
+        parsed_topic = ParsedTopic.from_topic(topic)
+        if parsed_topic is None:
+            log_debug("Ignoring installation ID handling - could not parse topic: %s", topic)
+            return
 
-    def _handle_normal_message(self, topic: str, payload: bytes) -> None:
+        self._installation_id = parsed_topic.installation_id
+        log_debug("Installation ID received: %s. Original topic: %s", self._installation_id, topic)
+        self._loop.call_soon_threadsafe(self._installation_id_event.set)
+
+    def _handle_normal_message(self, topic: str, payload: str, log_debug) -> None:
         """Handle regular MQTT message."""
         parsed_topic = ParsedTopic.from_topic(topic)
         if parsed_topic is None:
-            _LOGGER.debug("Ignoring message - could not parse topic: %s", topic)
+            log_debug("Ignoring message - could not parse topic: %s", topic)
             return
 
         fallback_to_metric_topic: bool = False
@@ -252,23 +309,26 @@ class Hub:
             desc_list = self.fallback_map.get(parsed_topic.wildcards_without_device_type)
             fallback_to_metric_topic = True
         if desc_list is None:
-            _LOGGER.debug("Ignoring message - no descriptor found for topic: %s", topic)
+            log_debug("Ignoring message - no descriptor found for topic: %s", topic)
             return
         if len(desc_list) == 1:
             desc = desc_list[0]
         else:
             desc = parsed_topic.match_from_list(desc_list)
             if desc is None:
-                _LOGGER.debug("Ignoring message - no matching descriptor found for list of topic: %s", topic)
+                log_debug("Ignoring message - no matching descriptor found for list of topic: %s", topic)
                 return
 
         device = self._get_or_create_device(parsed_topic, desc)
-        device.handle_message(fallback_to_metric_topic, topic, parsed_topic, desc, payload.decode(), self._loop, self)
+        tuple = device.handle_message(fallback_to_metric_topic, topic, parsed_topic, desc, payload, self._loop, self, log_debug)
+        if tuple:
+            self._new_metrics.append(tuple)
 
     async def disconnect(self) -> None:
         """Disconnect from the hub."""
         _LOGGER.info("Disconnecting from MQTT broker")
-        self._stop_keep_alive_loop()
+        self._stop_keepalive_loop()
+        await asyncio.sleep(0.1)
         if self._client is None:
             _LOGGER.debug("No client to disconnect")
             return
@@ -277,10 +337,10 @@ class Hub:
             _LOGGER.info("Disconnected from MQTT broker")
         self._client = None
 
-    async def _keep_alive(self) -> None:
+    async def _keepalive(self) -> None:
         """Send a keep alive message to the hub. Updates will only be made to the metrics
         for the 60 seconds following this method call."""
-        # cspell:disable-next-line
+        # Docuementation: https://github.com/victronenergy/dbus-flashmq
         keep_alive_topic = f"R/{self._installation_id}/keepalive"
 
         if self._client is None:
@@ -290,42 +350,43 @@ class Hub:
             _LOGGER.warning("Cannot send keepalive - client is not connected")
             return
         _LOGGER.debug("Sending keepalive message to topic: %s", keep_alive_topic)
-        self.publish(keep_alive_topic, b"1")
+        self._keepalive_counter += 1
+        keepalive_value = Hub.generate_keepalive_options(f"{self._client_id}-{self._keepalive_counter}")
+        self.publish(keep_alive_topic, keepalive_value)
 
     def publish(self, topic: str, value: PayloadType) -> None:
         assert self._client is not None
         prefixed_topic = self._add_topic_prefix(topic)
         self._client.publish(prefixed_topic, value)
 
-    async def _keep_alive_loop(self) -> None:
-        """Run keep_alive every 30 seconds."""
+    async def _keepalive_loop(self) -> None:
+        """Run keepalive every 30 seconds."""
         _LOGGER.info("Starting keepalive loop")
-        try:
-            while True:
-                try:
-                    await self._keep_alive()
-                    await asyncio.sleep(30)
-                except Exception as exc:
-                    _LOGGER.error("Error in keepalive loop: %s", exc, exc_info=True)
-                    await asyncio.sleep(5)  # Short delay before retrying
-        except asyncio.CancelledError:
-            _LOGGER.info("Keepalive loop canceled")
-            raise
+        while True:
+            try:
+                await self._keepalive()
+                await asyncio.sleep(30)
+            except asyncio.CancelledError:
+                _LOGGER.info("Keepalive loop canceled")
+                raise
+            except Exception as exc:
+                _LOGGER.error("Error in keepalive loop: %s", exc, exc_info=True)
+                await asyncio.sleep(5)  # Short delay before retrying
 
     def _start_keep_alive_loop(self) -> None:
         """Start the keep_alive loop."""
         _LOGGER.info("Creating keepalive task")
-        if self._keep_alive_task is None or self._keep_alive_task.done():
-            self._keep_alive_task = asyncio.create_task(self._keep_alive_loop())
+        if self._keepalive_task is None or self._keepalive_task.done():
+            self._keepalive_task = asyncio.create_task(self._keepalive_loop())
         else:
             _LOGGER.warning("Keepalive task already running")
 
-    def _stop_keep_alive_loop(self) -> None:
-        """Stop the keep_alive loop."""
-        if self._keep_alive_task is not None:
+    def _stop_keepalive_loop(self) -> None:
+        """Stop the keepalive loop."""
+        if self._keepalive_task is not None:
             _LOGGER.info("Cancelling keepalive task")
-            self._keep_alive_task.cancel()
-            self._keep_alive_task = None
+            self._keepalive_task.cancel()
+            self._keepalive_task = None
 
 
     async def create_full_raw_snapshot(self) -> dict:
@@ -341,8 +402,8 @@ class Hub:
         self._client.on_message = self._on_snapshot_message
         self._subscribe("#")
         _LOGGER.info("Subscribed to all topics for snapshot")
-        await self._keep_alive()
-        await self._wait_for_first_refresh()
+        await self._keepalive()
+        await self.wait_for_first_refresh()
         _LOGGER.info("Snapshot complete with %d top-level entries", len(self._snapshot))
         return self._snapshot
 
@@ -363,7 +424,7 @@ class Hub:
             if "full_publish_completed" in topic:
                 _LOGGER.info("Full publish completed, unsubscribing from notification")
                 if self._client is not None:
-                    self._unsubscribe("N/+/full_publish_completed")
+                    self._unsubscribe("#")
                 self._loop.call_soon_threadsafe(self._first_refresh_event.set)
                 return
 
@@ -476,10 +537,12 @@ class Hub:
             _LOGGER.error("Timeout waiting for first first connection")
             raise CannotConnectError("Timeout waiting for first connection")
 
-    async def _wait_for_first_refresh(self) -> None:
+    async def wait_for_first_refresh(self) -> None:
         """Wait for the first full refresh to complete."""
+        _LOGGER.info("Waiting for first refresh")
         try:
             await asyncio.wait_for(self._first_refresh_event.wait(), timeout=60)
+            _LOGGER.info("Devices and metrics initialized. Found %d devices", len(self._devices))
         except asyncio.TimeoutError:
             _LOGGER.error("Timeout waiting for first full refresh")
             raise CannotConnectError("Timeout waiting for first full refresh")
@@ -551,9 +614,10 @@ class Hub:
 
     def on_connect_fail(self, client: MQTTClient, userdata: Any) -> None:
         """Handle connection failure callback."""
-        _LOGGER.error("Connection to MQTT broker failed")
-        self._connected_failed = True
-        self._loop.call_soon_threadsafe(self._connected_event.set)
+        _LOGGER.warning("Connection to MQTT broker failed")
+        self._connected_failed_attempts += 1
+        if self._connected_failed_attempts >= CONNECT_MAX_FAILED_ATTEMPTS:
+            self._loop.call_soon_threadsafe(self._connected_event.set)
 
     @staticmethod
     def expand_topic_list(topic_list: list[TopicDescriptor]) -> list[TopicDescriptor]:
@@ -579,6 +643,32 @@ class Hub:
                 expanded.append(td)
         return expanded
 
+    @property
+    def on_new_metric(self) -> CallbackOnNewMetric | None:
+        """Returns the on_new_metric callback."""
+        return self._on_new_metric
+
+    @on_new_metric.setter
+    def on_new_metric(self, value: CallbackOnNewMetric):
+        """Sets the on_new_metric callback."""
+        self._on_new_metric = value
+
+    @staticmethod
+    def generate_keepalive_options(echo_value: str) -> str:
+        """Generate a JSON string for keepalive options with a configurable echo value."""
+        options = {
+            "keepalive-options": [
+                {"full-publish-completed-echo": echo_value}
+            ]
+        }
+        return json.dumps(options)
+
+    @staticmethod
+    def get_keepalive_echo(value: str) -> str:
+        """Extract the keepalive echo value from the published message."""
+        publish_completed_message = json.loads(value)
+        echo = publish_completed_message.get("full-publish-completed-echo", False)
+        return echo
 
 
 class CannotConnectError(Exception):
